﻿using Infragistics.Controls.Grids;
using Infragistics.DragDrop;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

namespace XamGridDragDropCase
{
    public class CustomXamGrid : XamGrid
    {
        #region Events

        public event EventHandler<RowDragDropEventArgs> RowDragCancel;

        public event EventHandler<RowDragDropEventArgs> RowDragEnd;

        public event EventHandler<RowDragDropEventArgs> RowDragEnter;

        public event EventHandler<RowDragDropEventArgs> RowDragLeave;

        public event EventHandler<RowDragDropEventArgs> RowDragOver;

        public event EventHandler<RowDragDropEventArgs> RowDragStart;

        public event EventHandler<RowDragDropEventArgs> RowDrop;

        #endregion // Events

        #region Properties

        #region RowDragTemplate
        /// <summary>
        /// Identifies the <see cref="RowDragTemplate"/> dependency property.
        /// </summary>
        public DataTemplate RowDragTemplate
        {
            get { return (DataTemplate)GetValue(RowDragTemplateProperty); }
            set { SetValue(RowDragTemplateProperty, value); }
        }

        /// <summary>
        /// Gets or sets data template used to visualize dragged row while drag-and-drop operation is performed. 
        /// This is a dependency property.
        /// </summary>
        public static readonly DependencyProperty RowDragTemplateProperty =
            DependencyProperty.Register(
            "RowDragTemplate",
            typeof(DataTemplate),
            typeof(CustomXamGrid),
            null);


        #endregion // RowDragTemplate

        #region RowDragChannels

        /// <summary>
        /// Identifies the <see cref="RowDragChannels"/> dependency property.
        /// </summary>
        public static readonly DependencyProperty RowDragChannelsProperty =
            DependencyProperty.Register(
            "RowDragChannels",
            typeof(ObservableCollection<string>),
            typeof(CustomXamGrid),
            null);

        /// <summary>
        /// Gets or sets the channels that rows can be dragged into.
        /// This is a dependency property.
        /// </summary>
        [TypeConverter(typeof(StringToDragDropChannelsCollectionConverter))]
        public ObservableCollection<string> RowDragChannels
        {
            get
            {
                ObservableCollection<string> dragChannels = (ObservableCollection<string>)this.GetValue(RowDragChannelsProperty);
                if (dragChannels == null)
                {
                    dragChannels = new ObservableCollection<string>();
                    this.SetValue(RowDragChannelsProperty, dragChannels);
                }

                return dragChannels;
            }

            set
            {
                this.SetValue(RowDragChannelsProperty, value);
            }
        }

        #endregion // RowDragChannels

        #endregion // Properties

        #region Methods

        #region Overrides

        #region OnCellControlAttached

        protected override bool OnCellControlAttached(Cell cell)
        {
            if (cell.Row.Control != null)
            {
                DragSource dragSource = DragDropManager.GetDragSource(cell.Row.Control);
                if (dragSource == null)
                {
                    dragSource = new DragSource { IsDraggable = true, DataObject = cell.Row.Data };

                    Binding dragTemplateBinding = new Binding("RowDragTemplate");
                    dragTemplateBinding.Source = this;
                    BindingOperations.SetBinding(dragSource, DragSource.DragTemplateProperty, dragTemplateBinding);

                    Binding dragChannelsBinding = new Binding("RowDragChannels");
                    dragChannelsBinding.Mode = BindingMode.TwoWay;
                    dragChannelsBinding.Source = this;
                    BindingOperations.SetBinding(dragSource, DragSource.DragChannelsProperty, dragChannelsBinding);

                    dragSource.DragCancel += this.OnRowDragCancel;
                    dragSource.DragEnd += this.OnRowDragEnd;
                    dragSource.DragEnter += this.OnRowDragEnter;
                    dragSource.DragLeave += this.OnRowDragLeave;
                    dragSource.DragOver += this.OnRowDragOver;
                    dragSource.DragStart += this.OnRowDragStart;
                    dragSource.Drop += this.OnRowDrop;

                    DragDropManager.SetDragSource(cell.Row.Control, dragSource);
                }
                else
                {
                    // update it to hold the new data
                    dragSource.DataObject = cell.Row.Data;
                }
            }

            return base.OnCellControlAttached(cell);
        }

        #endregion  // OnCellControlAttached

        #endregion // Overrides

        #region Private Methods
        private void OnRowDragCancel(object sender, DragDropEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDragCancel, sender, e);
        }

        private void OnRowDragEnd(object sender, DragDropEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDragEnd, sender, e);
        }

        private void OnRowDragEnter(object sender, DragDropCancelEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDragEnter, sender, e);
        }

        private void OnRowDragLeave(object sender, DragDropEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDragLeave, sender, e);
        }

        private void OnRowDragOver(object sender, DragDropMoveEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDragOver, sender, e);
        }

        private void OnRowDrop(object sender, DropEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDrop, sender, e);
        }

        private void OnRowDragStart(object sender, DragDropStartEventArgs e)
        {
            this.RaiseRowDragDropEvent(this.RowDragStart, sender, e);
        }

        private void RaiseRowDragDropEvent(EventHandler<RowDragDropEventArgs> eventHandler, object sender, DragDropEventArgs dragDropEventArgs)
        {
            if (eventHandler != null)
            {
                RowDragDropEventArgs args = new RowDragDropEventArgs((DragSource)sender, dragDropEventArgs);
                eventHandler(this, args);
            }
        }
        #endregion // Private Methods

        #endregion // Methods
    }

    public class RowDragDropEventArgs : EventArgs
    {
        public RowDragDropEventArgs(DragSource dragSource, DragDropEventArgs dragDropEventArgs)
        {
            this.DragSource = dragSource;
            this.DragDropEventArgs = dragDropEventArgs;
        }

        /// <summary>
        /// Gets the <see cref="DragDropEventArgs"/> passed originaly as event arguments of <see cref="DragSource"/> events.        
        /// </summary>
        /// <remarks>
        public DragDropEventArgs DragDropEventArgs
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the <see cref="DragSource"/> instance used in current drag-drop operation.
        /// </summary>
        public DragSource DragSource
        {
            get;
            private set;
        }
    }
}
